// -*- c++ -*-
//
// $Id: pstatus.h,v 1.1.1.1.2.1 2001/10/29 04:33:22 bbarrett Exp $
//
// Copyright 1997-2001, University of Notre Dame.
// Authors: Jeffrey M. Squyres, Jeremy G. Siek, Michael P. McNally, and
//          Andrew Lumsdaine
// 
// This file is part of the Notre Dame C++ bindings for MPI.
// 
// You should have received a copy of the License Agreement for the Notre
// Dame C++ bindings for MPI along with the software; see the file
// LICENSE.  If not, contact Office of Research, University of Notre
// Dame, Notre Dame, IN 46556.
// 
// Redistribution and use in source and binary forms, with or without
// modification, are permitted subject to the conditions specified in the
// LICENSE file.
// 
// THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
// IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
// WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
// DISCLAIMED. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT,
// INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
// (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
// SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
// HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
// STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
// IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
// POSSIBILITY OF SUCH DAMAGE.
// 
// Additional copyrights may follow.
//


class Status {
  friend class PMPI::Comm; //so I can access pmpi_status data member in comm.cc
  friend class PMPI::Request; //and also from request.cc

public:
  // construction
  inline Status() { }
  // copy
  Status(const Status& data) : mpi_status(data.mpi_status) { }

  inline Status(const MPI_Status &i) : mpi_status(i) { }

  inline virtual ~Status() {}

  Status& operator=(const Status& data) {
    mpi_status = data.mpi_status; return *this;
  } 

  // comparison, don't need for status

  // inter-language operability
  inline Status& operator= (const MPI_Status &i) {
    mpi_status = i; return *this; }
  inline operator MPI_Status () const { return mpi_status; }
  inline operator MPI_Status* () const { return (MPI_Status*)&mpi_status; }

  //
  // Point-to-Point Communication
  //

  inline virtual int Get_count(const Datatype& datatype) const;

  inline virtual MPI2CPP_BOOL_T Is_cancelled() const;

  virtual int Get_elements(const Datatype& datatype) const;

  //
  // Status Access
  //
  inline virtual int Get_source() const;

  inline virtual void Set_source(int source);
  
  inline virtual int Get_tag() const;

  inline virtual void Set_tag(int tag);
  
  inline virtual int Get_error() const;

  inline virtual void Set_error(int error);
  
protected:
  MPI_Status mpi_status;

};
