/*
 * Copyright 1998-2001, University of Notre Dame.
 * Authors: Jeffrey M. Squyres, Arun Rodrigues, and Brian Barrett with
 *          Kinis L. Meyer, M. D. McNally, and Andrew Lumsdaine
 * 
 * This file is part of the Notre Dame LAM implementation of MPI.
 * 
 * You should have received a copy of the License Agreement for the Notre
 * Dame LAM implementation of MPI along with the software; see the file
 * LICENSE.  If not, contact Office of Research, University of Notre
 * Dame, Notre Dame, IN 46556.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted subject to the conditions specified in the
 * LICENSE file.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
 * IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 * 
 * Additional copyrights may follow.
 * 
 *	Ohio Trollius
 *	Copyright 1997 The Ohio State University
 *	NJN
 *
 *	$Id: tgetconts_f.c,v 1.1 1999/09/02 01:34:34 prijks Exp $
 *
 *	Function:	- MPI_Type_get_contents F77 wrapper
 */

#include <errno.h>
#include <stdlib.h>

#include <lam_config.h>

#include <blktype.h>
#include <mpi.h>
#include <MPISYSF.h>
#include <mpisys.h>


void 
mpi_type_get_contents_(t, ni, na, nd, is, as, ds, ierr)

int			*t, *ni, *na, *nd, *is, *ds, *ierr;
MPI_Aint		*as;

{
	MPI_Datatype	*dtypes;
	int		i;

	dtypes = (MPI_Datatype *) malloc(*nd * sizeof(MPI_Datatype));
	if (dtypes == 0) {
		lam_setfunc_m(BLKMPITGETCONT);
		*ierr = lam_errfunc(MPI_COMM_WORLD, BLKMPITGETCONT,
					lam_mkerr(MPI_ERR_OTHER, errno));
		return;
	}

	*ierr = MPI_Type_get_contents(GETHDL(*t), *ni, *na, 
					*nd, is, as, dtypes);
	if (*ierr != MPI_SUCCESS) return;

	for (i = 0; i < *nd; i++) {
		if (dtypes[i]->dt_f77handle != -1) {
			ds[i] = dtypes[i]->dt_f77handle;
		} else {
			if (lam_F_make_hdl(&ds[i], ierr, 
						dtypes[i], BLKMPITGETCONT)) {
				dtypes[i]->dt_f77handle = ds[i];
			} else {
				return;
			}
		}
	}

	free((char *) dtypes);
}

