/*
 * Copyright 1998-2001, University of Notre Dame.
 * Authors: Jeffrey M. Squyres, Arun Rodrigues, and Brian Barrett with
 *          Kinis L. Meyer, M. D. McNally, and Andrew Lumsdaine
 * 
 * This file is part of the Notre Dame LAM implementation of MPI.
 * 
 * You should have received a copy of the License Agreement for the Notre
 * Dame LAM implementation of MPI along with the software; see the file
 * LICENSE.  If not, contact Office of Research, University of Notre
 * Dame, Notre Dame, IN 46556.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted subject to the conditions specified in the
 * LICENSE file.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
 * IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 * 
 * Additional copyrights may follow.
 * 
 *	Ohio Trollius
 *
 *	$Id: tfclose.c,v 6.3 1999/05/27 20:07:05 kmeyer1 Exp $
 * 
 *	We are just putting our stamp on this for housekeeping reasons.
 */

/*
 * This file is originally from dLibs 1.2 by Dale Schumacher et al.
 * It was modified for Trollius by MB, 88/12.
 * It was modified again for portability by GDB, 91/08.
 * And yet again by JRV for LAM, 93/05.
 */

#include <stdlib.h>
#include <tstdio.h>

#include <laminternal.h>


int
tfclose(fp)
	TFILE *fp;
{
	int f;

	if(fp == NULL)
		return(EOF);		/* NULL file pointer file */
	f = fp->_flag;
	if((f & (_TIOREAD | _TIOWRT)) == 0)
		return(EOF);		/* file not open! */
	tfflush(fp);
	if(fp->_flag & _TIOMYBUF)	/* throw away non-standard buffer */
	{
		free(fp->_base);
	}
	fp->_base = NULL;
	fp->_ptr = NULL;
	fp->_bsiz = 0;
	fp->_flag = 0;			/* clear status */
	if(f & _TIODEV)			/* leave tty's alone */
		return(0);

	f = lam_iomap_close(f, fp->_file);

	return(f ? EOF : 0);
}

/*
 *	fcloseall
 *
 *	Function:	- Close all streams.
 */

void
tfcloseall()

{
	int i, f;

	for(i=0; i<_TNFILE; ++i) {
		f = _tiob[i]._flag;
		if(f & (_TIOREAD | _TIOWRT))
			tfclose(&_tiob[i]);
	}
}
